/* sysSerial.c -  MPC860 SCC UART BSP serial device initialization */

/* Copyright 1984-1998 Wind River Systems, Inc. */
/* Copyright 1997,1998 Motorola, Inc., All Rights Reserved */
#include "copyright_wrs.h" 

/*
modification history
--------------------
---,21oct98,bcc  modified SCC3 and SCC4 to hold RTS low all the time
01b,20nov97,rhk  WRS code review cleanup.
01a,26jun97,srr  created from ads860, version 01d.
*/

/*
DESCRIPTION
This module provides code to initialize the serial port for the MBX.
Two calls exist for the inialization, sysSerialHwInit and 
sysSerialHwInit2.  They are called by sysHwInit and sysHwInit2, respectively
in sysLib.c.  The routine sysSerialHwInit sets up the necessary data
structures: PPC860SMC_CHAN and the tranmit and receive buffers. 
sysSerialHwInit2 connects the serial interrupt and enables it.

The routine sysSerialChanGet is called by bootConfig.c and usrConfig.c
as part of the generic serial driver setup ( when INCLUDE_TTY_DEV is
defined).  This instantiates a specific serial device configuration on
the MBX into the generic TTY driver.

The routine sysSerialReset is called by sysToMonitor in sysLib.c and will
reset the serial port.
*/

#include "vxWorks.h"
#include "iv.h"
#include "intLib.h"
#include "config.h"
#include "sysLib.h"


#if (SERIAL_PORT == SERIAL_PORT_SMC)

#include "ppc860Sio.h"
#include "ppc860Cpm.h"

static PPC860SMC_CHAN ppc860Chan;
#endif /* SERIAL_PORT == SERIAL_PORT_SMC */

#ifdef INCLUDE_SCC_SIO				/* using SCC channels */

#include "mbx800.h"
#include "ppc860SioScc.c"            /* Well, this is how kurtis did it ? */

/*** External Functions ***/
extern void ppc860SccDevInit();
extern void ppc860SccInt();


static PPC860SCC_CHAN ppc860Chan2;
static PPC860SCC_CHAN ppc860Chan3;
static PPC860SCC_CHAN ppc860Chan4;

#endif /* INCLUDE_SCC_SIO */

#if (SERIAL_PORT == SERIAL_PORT_COM)
#include "drv/sio/i8250Sio.h"

IMPORT void             sysOutByte (UINT32, UINT8);
IMPORT UCHAR            sysInByte (UINT32);

/* device initialization structure */

typedef struct
    {
    USHORT vector;                      /* Interrupt vector */
    ULONG  baseAdrs;                    /* Register base address */
    USHORT regSpace;                    /* Address Interval */
    } I8250_CHAN_PARAS;

/* forward declaration */

IMPORT int intEnable (int vector);

/* Local data structures */

static I8250_CHAN  i8250Chan[N_UART_CHANNELS];

static I8250_CHAN_PARAS devParas[] =
    {
       {COM1_INT_VEC, COM1_BASE_ADR, UART_REG_ADDR_INTERVAL},
    };

#define UART_REG(reg,chan) \
                (devParas[chan].baseAdrs + reg * devParas[chan].regSpace)

#endif /* SERIAL_PORT == SERIAL_PORT_COM */

#ifdef INCLUDE_KBD_LCD
#include "kbdLcd.h"
static KBDLCD_CHAN kbdlcdChan;
STATUS  result;
#endif

/******************************************************************************
*
* sysSerialHwInit - initialize the BSP serial devices to a quiesent state
*
* This routine initializes the BSP serial device descriptors and puts the
* devices in a quiesent state.  It is called from sysHwInit() with
* interrupts locked.
*
* Data Parameter Ram layout:
*
*.CS
* -----------------------------
* |                           | DPRAM base (address = 0x2000)
* |                           |
* |                           |
* |---------------------------|
* | 8 bytes per descriptor    | Tx Buffer Descriptor (0x2200)
* |---------------------------|
* |                           |
* |---------------------------|
* | 16 descriptors @          | Rx Buffer Descriptors (0x2210)
* | 8 bytes per descriptor    | 
* |                           |
* |---------------------------|
* |                           | end Rx BDs (0x2290)
* |                           |
* |                           |
* |---------------------------|
* | 80 bytes allowed          | Tx Buffer (0x2300)
* |---------------------------|
* | one receive char/buffer   | Rx Buffer (0x2380)
* |---------------------------|
* |                           |
* |                           |
* |                           |
* |                           |
* |---------------------------|
* | 3A hex bytes of parameter | SMC1 Parameter Area (0x3e80)
* | info including Rx and Tx  | 
* | BD pointers, func codes   |
* | etc...                    |
* |---------------------------|
*
*.CE
*
* RETURNS: N/A
*/ 

void sysSerialHwInit (void)
    {

#if (SERIAL_PORT == SERIAL_PORT_COM)
    int i;

    /* enable serial I/O using COM1 on the board */

    *BCSR1 |= BCSR1_RS232_COM1;		/* Use COM1 instead of SMC1 */
    *BCSR1 &= ~BCSR1_RS232_DT;          /* Enable Transceiver */	
#endif

#if (SERIAL_PORT == SERIAL_PORT_SMC)
    /* enable serial I/O using SCC1 on the board */

    *BCSR1 &= (~BCSR1_RS232_COM1 & ~BCSR1_RS232_DT);

    /* intialize the chips device descriptors */

    ppc860Chan.clockRate      = brgClkFreq;	    /* BRGCLK freq (Hz) */

    ppc860Chan.regBase        = vxImmrGet();  	    /* IMMR reg has base adr */
    ppc860Chan.bgrNum         = 1;                  /* use BRG1 */
    ppc860Chan.uart.smcNum    = 1;                  /* SMC1 wired for rs232 */
    ppc860Chan.uart.txBdNum   = 1;                  /* use 1 transmit BD */
    ppc860Chan.uart.rxBdNum   = 0x10;         /* use 16 receive BD */

    /* setup the transmit buffer descriptor base addr */

    ppc860Chan.uart.txBdBase  = (SMC_BUF *) TX_BUF_DESC_BA;

    /* setup the receive buffer descriptor base addr */

    ppc860Chan.uart.rxBdBase  = (SMC_BUF *) RX_BUF_DESC_BA;

    ppc860Chan.uart.txBufBase = (u_char *) 
		(MPC860_DPRAM_BASE(ppc860Chan.regBase) + TX_BUFFER);
                                                    /* tx buf base */
    ppc860Chan.uart.rxBufBase = (u_char *) 
		(MPC860_DPRAM_BASE(ppc860Chan.regBase) + RX_BUFFER);
                                                    /* rx buf base */

    /* WE MAY WANT TO INCREASE THE BUFFER SIZES. */

    ppc860Chan.uart.txBufSize = 0x1;                /* transmit buffer size */

    ppc860Chan.uart.pSmc = (SMC *) ((UINT32) 
	PPC860_DPR_SMC1( MPC860_DPRAM_BASE(ppc860Chan.regBase) ));
                                                    /* SMC1 DPRAM addr params */
    ppc860Chan.uart.pSmcReg = 
        (SMC_REG *) ((UINT32) MPC860_SMCMR1(ppc860Chan.regBase));
						    /* SMCMR1 for SMC1 */
    /* configure the port lines */

    *MPC860_PBPAR(ppc860Chan.regBase) |= PORT_B_PINS_SER;
    *MPC860_PBDIR(ppc860Chan.regBase) &= ~PORT_B_PINS_SER;
    *MPC860_PBODR(ppc860Chan.regBase) &= ~PORT_B_PINS_SER;

    ppc860Chan.pBaud = (UINT32 *) MPC860_BRGC1(ppc860Chan.regBase);
    ppc860Chan.channelMode = 0;

    *MPC860_SDCR(ppc860Chan.regBase) = 1;           /* */

    /* reset the chip */

    ppc860DevInit(&ppc860Chan);
#endif /* SERIAL_PORT == SERIAL_PORT_SMC */

#ifdef INCLUDE_SCC_SIO

    /* enable serial I/O using SCC2 on the board */

  /*  *BCSR1 |= ~BCSR1_RS232_COM1; */

    /* intialize the chips device descriptors */

    ppc860Chan2.clockRate      = brgClkFreq;	    /*  BRGCLK freq (Hz) */

    ppc860Chan2.regBase        = vxImmrGet();  	    /* IMMR reg has base adr */
    ppc860Chan2.bgrNum         = 2;                  /* use BRG2 */
    ppc860Chan2.uart.sccNum    = 2;                  /* SCC2 wired for rs232 */
    ppc860Chan2.uart.txBdNum   = 1;                  /* use 1 transmit BD */
    ppc860Chan2.uart.rxBdNum   = 0x10;                /* use 16 receive BD */
    ppc860Chan2.uart.txBdBase  = (SCC_BUF *) 0x2208; /* transmit BD base adrs */
    ppc860Chan2.uart.rxBdBase  = (SCC_BUF *) 0x2a40;  /* receive BD base adrs */
    ppc860Chan2.uart.txBufBase = (u_char *) 
		(MPC860_DPRAM_BASE(ppc860Chan2.regBase) + 0xbc0);       
                                                    /* tx buf base */
    ppc860Chan2.uart.rxBufBase = (u_char *) 
		(MPC860_DPRAM_BASE(ppc860Chan2.regBase) + 0xc40);
                                                    /* rx buf base */

/*********************************************/
/* WE MAY WANT TO INCREASE THE BUFFER SIZES. */
/*********************************************/

    ppc860Chan2.uart.txBufSize = 0x1;                /* transmit buffer size */
    ppc860Chan2.uart.pScc = (SCC *) ((UINT32) 
	PPC860_DPR_SCC2( MPC860_DPRAM_BASE(ppc860Chan2.regBase) ));
                                                 /* DPRAM addr of SCC2 params */
    ppc860Chan2.uart.pSccReg = 
        (SCC_REG *) ((UINT32) MPC860_GSMR_L2(ppc860Chan2.regBase));
						    /* SMCMR2 for SCC2 */

    ppc860Chan2.pBaud = (UINT32 *) MPC860_BRGC2(ppc860Chan2.regBase);
    ppc860Chan2.channelMode = 0;

 /*   *MPC860_PBPAR(ppc860Chan.regBase) = 0xC0;        select RS232 pins */

    *MPC860_PAPAR(ppc860Chan2.regBase) |= 0xC;
    *MPC860_PADIR(ppc860Chan2.regBase) &= 0xfff3;
    *MPC860_PAODR(ppc860Chan2.regBase) &= 0xfff7;
    
    *MPC860_PCPAR(ppc860Chan2.regBase) &= 0xbcff;

  /**** This pin configuration will drive CTS and CD low! *****
   **** If CTS and CD are used for framing, they must be changed ****/
    *MPC860_PCDIR(ppc860Chan2.regBase) |= ((1<<(15-8))|(1<<(15-9)));    
    *MPC860_PCDAT(ppc860Chan2.regBase) &= ~((1<<(15-8))|(1<<(15-9)));    
    *MPC860_PCSO(ppc860Chan2.regBase) |= ((1<<(15-8))|(1<<(15-9)));


    *MPC860_SDCR(ppc860Chan2.regBase) = 1;           /* */

    /* reset the chip */

    ppc860SccDevInit(&ppc860Chan2);

    /* enable serial I/O using SCC3 on the board */

  /*  *BCSR1 |= ~BCSR1_RS232_COM1; */

    /* intialize the chips device descriptors */

    ppc860Chan3.clockRate      = brgClkFreq;	    /*  BRGCLK freq (Hz) */

    ppc860Chan3.regBase        = vxImmrGet();  	    /* IMMR reg has base adr */
    ppc860Chan3.bgrNum         = 3;                  /* use BRG2 */
    ppc860Chan3.uart.sccNum    = 3;                  /* SCC3 wired for rs232 */
    ppc860Chan3.uart.txBdNum   = 1;                  /* use 1 transmit BD */
    ppc860Chan3.uart.rxBdNum   = 0x10;  /*use 8 receive BD (bsp bd conflict??)*/
    ppc860Chan3.uart.txBdBase  = (SCC_BUF *) 0x2290; /*transmit BD base adrs */
    ppc860Chan3.uart.rxBdBase  = (SCC_BUF *) 0x2ac0;  /* receive BD base adrs */
    ppc860Chan3.uart.txBufBase = (u_char *) 
		(MPC860_DPRAM_BASE(ppc860Chan3.regBase) + 0xcc0 );  
                                                    /* tx buf base */
    ppc860Chan3.uart.rxBufBase = (u_char *) 
		(MPC860_DPRAM_BASE(ppc860Chan3.regBase) + 0xd40);
                                                    /* rx buf base */

/*********************************************/
/* WE MAY WANT TO INCREASE THE BUFFER SIZES. */
/*********************************************/

    ppc860Chan3.uart.txBufSize = 0x1;                /* transmit buffer size */
    ppc860Chan3.uart.pScc = (SCC *) ((UINT32) 
	PPC860_DPR_SCC3( MPC860_DPRAM_BASE(ppc860Chan3.regBase) ));
                                                 /* DPRAM addr of SCC3 params */
    ppc860Chan3.uart.pSccReg = 
        (SCC_REG *) ((UINT32) MPC860_GSMR_L3(ppc860Chan3.regBase));
						    /* SMCMR2 for SCC3 */

    ppc860Chan3.pBaud = (UINT32 *) MPC860_BRGC3(ppc860Chan3.regBase);
    ppc860Chan3.channelMode = 0;

 /*   *MPC860_PBPAR(ppc860Chan.regBase) = 0xC0;        select RS232 pins */

    *MPC860_PDPAR(ppc860Chan3.regBase) |= ((1<<(15-10))|(1<<(15-11)));
    *MPC860_PDDIR(ppc860Chan3.regBase) &= ~(1<<(15-11));
    *MPC860_PDDIR(ppc860Chan3.regBase) |= (1<<(15-10));    

  /**** This pin configuration will drive CTS and CD low! *****
   **** If CTS and CD are used for framing, they must be changed ****/

 /*   *MPC860_PCPAR(ppc860Chan3.regBase) &= ~((1<<(15-7))|(1<<(15-6))); */
    *MPC860_PCDIR(ppc860Chan3.regBase) |= ((1<<(15-7))|(1<<(15-6)));    
    *MPC860_PCDAT(ppc860Chan3.regBase) &= ~((1<<(15-7))|(1<<(15-6)));    
    *MPC860_PCSO(ppc860Chan3.regBase) |= ((1<<(15-7))|(1<<(15-6)));
 
       /***** Drive RTS3 low all the time *****/
    *MPC860_PDDIR(ppc860Chan3.regBase) |= (1<<(15-7));    
    *MPC860_PDDAT(ppc860Chan3.regBase) &= ~(1<<(15-7));    

 /*   *MPC860_PCDIR(ppc860Chan3.regBase) &= ~((1<<(15-7))|(1<<(15-6)));  */  

    *MPC860_SDCR(ppc860Chan3.regBase) = 1;           /* */

    /* reset the chip */

    ppc860SccDevInit(&ppc860Chan3);

    /* enable serial I/O using SCC4 on the board */

  /*  *BCSR1 |= ~BCSR1_RS232_COM1; */

    /* intialize the chips device descriptors */

    ppc860Chan4.clockRate      = brgClkFreq;	    /*  BRGCLK freq (Hz) */

    ppc860Chan4.regBase        = vxImmrGet();  	    /* IMMR reg has base adr */
    ppc860Chan4.bgrNum         = 4;                  /* use BRG2 */
    ppc860Chan4.uart.sccNum    = 4;                  /* SCC4 wired for rs232 */
    ppc860Chan4.uart.txBdNum   = 1;                  /* use 1 transmit BD */
    ppc860Chan4.uart.rxBdNum   = 0x10;	      	     /* use 16 receive BD */
    ppc860Chan4.uart.txBdBase  = (SCC_BUF *) 0x2298; /*  transmit BD base adrs */
    ppc860Chan4.uart.rxBdBase  = (SCC_BUF *) 0x2b40 ;  /* receive BD base adrs */
    ppc860Chan4.uart.txBufBase = (u_char *) 
		(MPC860_DPRAM_BASE(ppc860Chan4.regBase) + 0xdc0);       
                                                    /* tx buf base */
    ppc860Chan4.uart.rxBufBase = (u_char *) 
		(MPC860_DPRAM_BASE(ppc860Chan.regBase) + 0xe40 );
                                                    /* rx buf base */

/*********************************************/
/* WE MAY WANT TO INCREASE THE BUFFER SIZES. */
/*********************************************/

    ppc860Chan4.uart.txBufSize = 0x1;                /* transmit buffer size */
    ppc860Chan4.uart.pScc = (SCC *) ((UINT32) 
	PPC860_DPR_SCC4( MPC860_DPRAM_BASE(ppc860Chan4.regBase) ));
                                                 /* DPRAM addr of SCC4 params */
    ppc860Chan4.uart.pSccReg = 
        (SCC_REG *) ((UINT32) MPC860_GSMR_L4(ppc860Chan4.regBase));
						    /* SMCMR2 for SCC2 */

    ppc860Chan4.pBaud = (UINT32 *) MPC860_BRGC4(ppc860Chan4.regBase);
    ppc860Chan4.channelMode = 0;

 /*   *MPC860_PBPAR(ppc860Chan.regBase) = 0xC0;        select RS232 pins */

    *MPC860_PDPAR(ppc860Chan4.regBase) |= ((1<<(15-8))|(1<<(15-9)));;
    *MPC860_PDDIR(ppc860Chan4.regBase) &= ~(1<<(15-9));
    *MPC860_PDDIR(ppc860Chan4.regBase) |= (1<<(15-8));    
    

    *MPC860_PCPAR(ppc860Chan4.regBase) &= ~((1<<(15-5))|(1<<(15-4)));

 /*****   *MPC860_PCDIR(ppc860Chan4.regBase) &= ~((1<<(15-5))|(1<<(15-4)));    
   *MPC860_PCDIR(ppc860Chan4.regBase) &= ~(1<<(15-5));   
     *MPC860_PCDIR(ppc860Chan4.regBase) |= (1<<(15-4));  *****/


  /**** This pin configuration will drive CTS and CD low! *****
   **** If CTS and CD are used for framing, they must be changed ****/

    *MPC860_PCDIR(ppc860Chan4.regBase) |= ((1<<(15-5))|(1<<(15-4)));    
    *MPC860_PCDAT(ppc860Chan4.regBase) &= ~((1<<(15-4))|(1<<(15-5)));  
    *MPC860_PCSO(ppc860Chan4.regBase) |= ((1<<(15-5))|(1<<(15-4)));
 
       /***** Drive RTS4 low all the time *****/
    *MPC860_PDDIR(ppc860Chan4.regBase) |= (1<<(15-6));    
    *MPC860_PDDAT(ppc860Chan4.regBase) &= ~(1<<(15-6));    



    *MPC860_SDCR(ppc860Chan4.regBase) = 1;           /* */

    /* reset the chip */

    ppc860SccDevInit(&ppc860Chan4);


#endif 


#if (SERIAL_PORT == SERIAL_PORT_COM)
    for (i = 0; i < N_UART_CHANNELS; i++)
        {
	i8250Chan[i].int_vec	 = devParas[i].vector;
	i8250Chan[i].channelMode = 0;
	i8250Chan[i].lcr	 = UART_REG(UART_LCR, i);
	i8250Chan[i].data	 = UART_REG(UART_RDR, i);
	i8250Chan[i].brdl	 = UART_REG(UART_BRDL, i);
	i8250Chan[i].brdh	 = UART_REG(UART_BRDH, i);
	i8250Chan[i].ier	 = UART_REG(UART_IER, i);
	i8250Chan[i].iid	 = UART_REG(UART_IID, i);
	i8250Chan[i].mdc	 = UART_REG(UART_MDC, i);
	i8250Chan[i].lst	 = UART_REG(UART_LST, i);
	i8250Chan[i].msr	 = UART_REG(UART_MSR, i);
	i8250Chan[i].outByte	 = (void (*) (int, char))  sysOutByte;
	i8250Chan[i].inByte	 = (UCHAR (*) ()) sysInByte;

	i8250HrdInit (&i8250Chan[i]);
        }

#endif /* SERIAL_PORT == SERIAL_PORT_COM */

#ifdef INCLUDE_KBD_LCD
    kbdInit(&kbdlcdChan); 	/* Initialize the channel structure */
    result = lcdInit();		/* initialize the LCD interface */
#endif

   }

/******************************************************************************
*
* sysSerialHwInit2 - connect BSP serial device interrupts
*
* This routine connects the BSP serial device interrupts.  It is called from
* sysHwInit2().  Serial device interrupts could not be connected in
* sysSerialHwInit() because the kernel memory allocator was not initialized
* at that point, and intConnect() calls malloc().
*
* RETURNS: N/A
*/ 

void sysSerialHwInit2 (void)
    {

    /* connect serial interrupts */

#if (SERIAL_PORT == SERIAL_PORT_SMC)
    (void) intConnect (IV_SMC1, (VOIDFUNCPTR) ppc860Int, (int) &ppc860Chan);

    *CIMR(vxImmrGet()) |= CIMR_SMC1;
#endif /* SERIAL_PORT == SERIAL_PORT_SMC */

#if (SERIAL_PORT == SERIAL_PORT_COM)

    int i;

    /* connect serial interrupts */

     for (i = 0; i < N_UART_CHANNELS; i++)
         if (i8250Chan[i].int_vec)
	     {
             (void) intConnect (INUM_TO_IVEC ((int ) i8250Chan[i].int_vec),
						i8250Int, (int)&i8250Chan[i] );

             intEnable (devParas[i].vector); 
             }
#endif /* SERIAL_PORT == SERIAL_PORT_COM */

#ifdef INCLUDE_SCC_SIO
         /* connect serial interrupts */

    (void) intConnect (IV_SCC2, (VOIDFUNCPTR) ppc860SccInt, (int) &ppc860Chan2);
    *CIMR(vxImmrGet()) |= CIMR_SCC2;

     (void) intConnect(IV_SCC3, (VOIDFUNCPTR) ppc860SccInt, (int) &ppc860Chan3);
    *CIMR(vxImmrGet()) |= CIMR_SCC3;

     (void) intConnect(IV_SCC4, (VOIDFUNCPTR) ppc860SccInt, (int) &ppc860Chan4);
    *CIMR(vxImmrGet()) |= CIMR_SCC4;
    *CICR(vxImmrGet()) &= 0xff00ffff;
    *CICR(vxImmrGet()) |= CICR_SCCAP_SCC1 | CICR_SCCBP_SCC2 | 
                          CICR_SCCCP_SCC3 | CICR_SCCDP_SCC4;
    
#endif

#ifdef INCLUDE_KBD_LCD
    (void) intConnect(INUM_TO_IVEC(KBD_INT_VEC), kbdIntr, (int)&kbdlcdChan);
    intEnable(KBD_INT_VEC);
    kbdHrdInit();
    result = lcdOpen(); /* if enabled, open the LCD driver now */
#endif
    }

/******************************************************************************
*
* sysSerialChanGet - get the SIO_CHAN device associated with a serial channel
*
* This routine gets the SIO_CHAN device associated with a specified serial
* channel.
*
* RETURNS: A pointer to the SIO_CHAN structure for the channel, or ERROR
* if the channel is invalid.
*/

SIO_CHAN * sysSerialChanGet
    (
    int channel		/* serial channel */
    )
    {
    switch (channel)
	{
	case SMC1_COM1_NUM:
#if (SERIAL_PORT == SERIAL_PORT_SMC)
            return ((SIO_CHAN *) &ppc860Chan);
#endif
#if (SERIAL_PORT == SERIAL_PORT_COM)
            return ((SIO_CHAN *) &i8250Chan[channel]);
#endif
#ifdef INCLUDE_SCC_SIO
         case 1: 
                return ((SIO_CHAN *) &ppc860Chan2);

         case 2: 
                return ((SIO_CHAN *) &ppc860Chan3);

         case 3: 
                return ((SIO_CHAN *) &ppc860Chan4);
#endif

#ifdef INCLUDE_KBD_LCD
        case KBD_LCD_NUM:
	    return ((SIO_CHAN *) &kbdlcdChan); 
#endif
	default:
	    return ((SIO_CHAN *) ERROR);
        }
    }

/*******************************************************************************
*
* sysSerialReset - reset the serial device 
*
* This function calls sysSerialHwInit() to reset the serial device
*
* RETURNS: N/A
*
*/

void sysSerialReset (void)
    {
#ifdef INCLUDE_KBD_LCD
    /* if the LCD interface is enabled, close the driver here */

    result = lcdClose();
#endif
    sysSerialHwInit ();
    }

